@@ -1,12 +1,18 @@
-import asyncio
+from __future__ import annotations
+
+import functools
 import hashlib
 import json
-import logging
 import os.path
 import re
+import secrets
+import sys
+from collections.abc import Callable
+from collections.abc import Sequence
 from io import BytesIO
-from itertools import islice
-from typing import ClassVar, Optional, Sequence, Union
+from typing import Any
+from typing import ClassVar
+from typing import Concatenate
 
 import tornado.escape
 import tornado.web
@@ -14,21 +20,38 @@ import tornado.websocket
 
 import mitmproxy.flow
 import mitmproxy.tools.web.master
-from mitmproxy import certs, command, contentviews
+import mitmproxy_rs
+from mitmproxy import certs
+from mitmproxy import command
+from mitmproxy import contentviews
 from mitmproxy import flowfilter
 from mitmproxy import http
 from mitmproxy import io
 from mitmproxy import log
 from mitmproxy import optmanager
 from mitmproxy import version
+from mitmproxy.dns import DNSFlow
 from mitmproxy.http import HTTPFlow
-from mitmproxy.tcp import TCPFlow, TCPMessage
+from mitmproxy.net.http import status_codes
+from mitmproxy.tcp import TCPFlow
+from mitmproxy.tcp import TCPMessage
+from mitmproxy.tools.web.webaddons import WebAuth
+from mitmproxy.udp import UDPFlow
+from mitmproxy.udp import UDPMessage
+from mitmproxy.utils import asyncio_utils
 from mitmproxy.utils.emoji import emoji
 from mitmproxy.utils.strutils import always_str
+from mitmproxy.utils.strutils import cut_after_n_lines
 from mitmproxy.websocket import WebSocketMessage
 
+TRANSPARENT_PNG = (
+    b"\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x01\x00\x00\x00\x01\x08"
+    b"\x04\x00\x00\x00\xb5\x1c\x0c\x02\x00\x00\x00\x0bIDATx\xdac\xfc\xff\x07"
+    b"\x00\x02\x00\x01\xfc\xa8Q\rh\x00\x00\x00\x00IEND\xaeB`\x82"
+)
 
-def cert_to_json(certs: Sequence[certs.Cert]) -> Optional[dict]:
+
+def cert_to_json(certs: Sequence[certs.Cert]) -> dict | None:
     if not certs:
         return None
     cert = certs[0]
@@ -40,7 +63,7 @@ def cert_to_json(certs: Sequence[certs.Cert]) -> Optional[dict]:
         "serial": str(cert.serial),
         "subject": cert.subject,
         "issuer": cert.issuer,
-        "altnames": cert.altnames,
+        "altnames": [str(x.value) for x in cert.altnames],
     }
 
 
@@ -59,6 +82,7 @@ def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
         "modified": flow.modified(),
         "marked": emoji.get(flow.marked, "🔴") if flow.marked else "",
         "comment": flow.comment,
+        "timestamp_created": flow.timestamp_created,
     }
 
     if flow.client_conn:
@@ -98,8 +122,8 @@ def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
         f["error"] = flow.error.get_state()
 
     if isinstance(flow, HTTPFlow):
-        content_length: Optional[int]
-        content_hash: Optional[str]
+        content_length: int | None
+        content_hash: str | None
 
         if flow.request.raw_content is not None:
             content_length = len(flow.request.raw_content)
@@ -139,26 +163,36 @@ def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
                 "timestamp_end": flow.response.timestamp_end,
             }
             if flow.response.data.trailers:
-                f["response"]["trailers"] = tuple(flow.response.data.trailers.items(True))
+                f["response"]["trailers"] = tuple(
+                    flow.response.data.trailers.items(True)
+                )
 
         if flow.websocket:
             f["websocket"] = {
                 "messages_meta": {
-                    "contentLength": sum(len(x.content) for x in flow.websocket.messages),
+                    "contentLength": sum(
+                        len(x.content) for x in flow.websocket.messages
+                    ),
                     "count": len(flow.websocket.messages),
-                    "timestamp_last": flow.websocket.messages[-1].timestamp if flow.websocket.messages else None,
+                    "timestamp_last": flow.websocket.messages[-1].timestamp
+                    if flow.websocket.messages
+                    else None,
                 },
                 "closed_by_client": flow.websocket.closed_by_client,
                 "close_code": flow.websocket.close_code,
                 "close_reason": flow.websocket.close_reason,
                 "timestamp_end": flow.websocket.timestamp_end,
             }
-    elif isinstance(flow, TCPFlow):
+    elif isinstance(flow, (TCPFlow, UDPFlow)):
         f["messages_meta"] = {
             "contentLength": sum(len(x.content) for x in flow.messages),
             "count": len(flow.messages),
             "timestamp_last": flow.messages[-1].timestamp if flow.messages else None,
         }
+    elif isinstance(flow, DNSFlow):
+        f["request"] = flow.request.to_json()
+        if flow.response:
+            f["response"] = flow.response.to_json()
 
     return f
 
@@ -167,7 +201,7 @@ def logentry_to_json(e: log.LogEntry) -> dict:
     return {
         "id": id(e),  # we just need some kind of id.
         "message": e.msg,
-        "level": e.level
+        "level": e.level,
     }
 
 
@@ -175,10 +209,68 @@ class APIError(tornado.web.HTTPError):
     pass
 
 
-class RequestHandler(tornado.web.RequestHandler):
-    application: "Application"
+class AuthRequestHandler(tornado.web.RequestHandler):
+    AUTH_COOKIE_NAME = "mitmproxy-auth"
+    AUTH_COOKIE_VALUE = b"y"
 
-    def write(self, chunk: Union[str, bytes, dict, list]):
+    def __init_subclass__(cls, **kwargs):
+        """Automatically wrap all request handlers with `_require_auth`."""
+        for method in cls.SUPPORTED_METHODS:
+            method = method.lower()
+            fn = getattr(cls, method)
+            if fn is not tornado.web.RequestHandler._unimplemented_method:
+                setattr(cls, method, AuthRequestHandler._require_auth(fn))
+
+    def auth_fail(self, invalid_password: bool) -> None:
+        """
+        Will be called when returning a 403.
+        May write a login form as the response.
+        """
+
+    @staticmethod
+    def _require_auth[**P, R](
+        fn: Callable[Concatenate[AuthRequestHandler, P], R],
+    ) -> Callable[Concatenate[AuthRequestHandler, P], R | None]:
+        @functools.wraps(fn)
+        def wrapper(
+            self: AuthRequestHandler, *args: P.args, **kwargs: P.kwargs
+        ) -> R | None:
+            if not self.current_user:
+                password = ""
+                if auth_header := self.request.headers.get("Authorization"):
+                    auth_scheme, _, auth_params = auth_header.partition(" ")
+                    if auth_scheme == "Bearer":
+                        password = auth_params
+
+                if not password:
+                    password = self.get_argument("token", default="")
+
+                if not self.settings["is_valid_password"](password):
+                    self.set_status(403)
+                    self.auth_fail(bool(password))
+                    return None
+                self.set_signed_cookie(
+                    self.AUTH_COOKIE_NAME,
+                    self.AUTH_COOKIE_VALUE,
+                    expires_days=400,
+                    httponly=True,
+                    samesite="Strict",
+                )
+            return fn(self, *args, **kwargs)
+
+        return wrapper
+
+    def get_current_user(self) -> bool:
+        return (
+            self.get_signed_cookie(self.AUTH_COOKIE_NAME, min_version=2)
+            == self.AUTH_COOKIE_VALUE
+        )
+
+
+class RequestHandler(AuthRequestHandler):
+    application: Application
+
+    def write(self, chunk: str | bytes | dict | list):
         # Writing arrays on the top level is ok nowadays.
         # http://flask.pocoo.org/docs/0.11/security/#json-security
         if isinstance(chunk, list):
@@ -196,17 +288,19 @@ class RequestHandler(tornado.web.RequestHandler):
             "Content-Security-Policy",
             "default-src 'self'; "
             "connect-src 'self' ws:; "
-            "style-src   'self' 'unsafe-inline'"
+            "style-src   'self' 'unsafe-inline'",
         )
 
     @property
     def json(self):
-        if not self.request.headers.get("Content-Type", "").startswith("application/json"):
+        if not self.request.headers.get("Content-Type", "").startswith(
+            "application/json"
+        ):
             raise APIError(400, "Invalid Content-Type, expected application/json.")
         try:
             return json.loads(self.request.body.decode())
         except Exception as e:
-            raise APIError(400, "Malformed JSON: {}".format(str(e)))
+            raise APIError(400, f"Malformed JSON: {str(e)}")
 
     @property
     def filecontents(self):
@@ -220,11 +314,11 @@ class RequestHandler(tornado.web.RequestHandler):
             return self.request.body
 
     @property
-    def view(self) -> "mitmproxy.addons.view.View":
+    def view(self) -> mitmproxy.addons.view.View:
         return self.application.master.view
 
     @property
-    def master(self) -> "mitmproxy.tools.web.master.WebMaster":
+    def master(self) -> mitmproxy.tools.web.master.WebMaster:
         return self.application.master
 
     @property
@@ -245,38 +339,66 @@ class RequestHandler(tornado.web.RequestHandler):
 
 
 class IndexHandler(RequestHandler):
+    def _is_fetch_mode_navigate(self) -> bool:
+        # Forbid access for non-navigate fetch modes so that they can't obtain xsrf_token.
+        return self.request.headers.get("Sec-Fetch-Mode", "navigate") == "navigate"
+
+    def auth_fail(self, invalid_password: bool) -> None:
+        # For mitmweb, we only write a login form for IndexHandler,
+        # which has additional Sec-Fetch-Mode protections.
+        if self._is_fetch_mode_navigate():
+            self.render("login.html", invalid_password=invalid_password)
+
     def get(self):
-        token = self.xsrf_token  # https://github.com/tornadoweb/tornado/issues/645
-        assert token
-        self.render("index.html")
+        # Forbid access for non-navigate fetch modes so that they can't obtain xsrf_token.
+        if self._is_fetch_mode_navigate():
+            self.render("index.html", xsrf_token=self.xsrf_token)
+        else:
+            raise APIError(
+                status_codes.PRECONDITION_FAILED,
+                f"Unexpected Sec-Fetch-Mode header: {self.request.headers.get('Sec-Fetch-Mode')}",
+            )
+
+    post = get  # login form
 
 
 class FilterHelp(RequestHandler):
     def get(self):
-        self.write(dict(
-            commands=flowfilter.help
-        ))
+        self.write(dict(commands=flowfilter.help))
 
 
-class WebSocketEventBroadcaster(tornado.websocket.WebSocketHandler):
+class WebSocketEventBroadcaster(tornado.websocket.WebSocketHandler, AuthRequestHandler):
     # raise an error if inherited class doesn't specify its own instance.
-    connections: ClassVar[set]
+    connections: ClassVar[set[WebSocketEventBroadcaster]]
 
-    def open(self):
+    def open(self, *args, **kwargs):
         self.connections.add(self)
 
     def on_close(self):
-        self.connections.remove(self)
+        self.connections.discard(self)
+
+    @classmethod
+    def send(cls, conn: WebSocketEventBroadcaster, message: bytes) -> None:
+        async def wrapper():
+            try:
+                await conn.write_message(message)
+            except tornado.websocket.WebSocketClosedError:
+                cls.connections.discard(conn)
+
+        asyncio_utils.create_task(
+            wrapper(),
+            name="WebSocketEventBroadcaster",
+            keep_ref=True,
+        )
 
     @classmethod
     def broadcast(cls, **kwargs):
-        message = json.dumps(kwargs, ensure_ascii=False).encode("utf8", "surrogateescape")
+        message = json.dumps(kwargs, ensure_ascii=False).encode(
+            "utf8", "surrogateescape"
+        )
 
-        for conn in cls.connections:
-            try:
-                conn.write_message(message)
-            except Exception:  # pragma: no cover
-                logging.error("Error sending message", exc_info=True)
+        for conn in cls.connections.copy():
+            cls.send(conn, message)
 
 
 class ClientConnection(WebSocketEventBroadcaster):
@@ -289,23 +411,35 @@ class Flows(RequestHandler):
 
 
 class DumpFlows(RequestHandler):
-    def get(self):
+    def get(self) -> None:
         self.set_header("Content-Disposition", "attachment; filename=flows")
         self.set_header("Content-Type", "application/octet-stream")
 
-        bio = BytesIO()
-        fw = io.FlowWriter(bio)
-        for f in self.view:
-            fw.add(f)
+        match: Callable[[mitmproxy.flow.Flow], bool]
+        try:
+            match = flowfilter.parse(self.request.arguments["filter"][0].decode())
+        except ValueError:  # thrown py flowfilter.parse if filter is invalid
+            raise APIError(400, f"Invalid filter argument / regex")
+        except (
+            KeyError,
+            IndexError,
+        ):  # Key+Index: ["filter"][0] can fail, if it's not set
 
-        self.write(bio.getvalue())
-        bio.close()
+            def match(_) -> bool:
+                return True
 
-    def post(self):
+        with BytesIO() as bio:
+            fw = io.FlowWriter(bio)
+            for f in self.view:
+                if match(f):
+                    fw.add(f)
+            self.write(bio.getvalue())
+
+    async def post(self):
         self.view.clear()
         bio = BytesIO(self.filecontents)
-        for i in io.FlowReader(bio).stream():
-            asyncio.ensure_future(self.master.load_flow(i))
+        for f in io.FlowReader(bio).stream():
+            await self.master.load_flow(f)
         bio.close()
 
 
@@ -351,7 +485,7 @@ class FlowHandler(RequestHandler):
             self.flow.kill()
         self.view.remove([self.flow])
 
-    def put(self, flow_id):
+    def put(self, flow_id) -> None:
         flow: mitmproxy.flow.Flow = self.flow
         flow.backup()
         try:
@@ -403,6 +537,8 @@ class FlowHandler(RequestHandler):
                             raise APIError(400, f"Unknown update response.{k}: {v}")
                 elif a == "marked":
                     flow.marked = b
+                elif a == "comment":
+                    flow.comment = b
                 else:
                     raise APIError(400, f"Unknown update {a}: {b}")
         except APIError:
@@ -439,13 +575,13 @@ class FlowContent(RequestHandler):
 
     def get(self, flow_id, message):
         message = getattr(self.flow, message)
+        assert isinstance(self.flow, HTTPFlow)
 
         original_cd = message.headers.get("Content-Disposition", None)
         filename = None
         if original_cd:
-            filename = re.search(r'filename=([-\w" .()]+)', original_cd)
-            if filename:
-                filename = filename.group(1)
+            if m := re.search(r'filename=([-\w" .()]+)', original_cd):
+                filename = m.group(1)
         if not filename:
             filename = self.flow.request.path.split("?")[0].split("/")[-1]
 
@@ -461,25 +597,34 @@ class FlowContent(RequestHandler):
 class FlowContentView(RequestHandler):
     def message_to_json(
         self,
-        viewname: str,
-        message: Union[http.Message, TCPMessage, WebSocketMessage],
-        flow: Union[HTTPFlow, TCPFlow],
-        max_lines: Optional[int] = None
+        view_name: str,
+        message: http.Message | TCPMessage | UDPMessage | WebSocketMessage,
+        flow: HTTPFlow | TCPFlow | UDPFlow,
+        max_lines: int | None = None,
+        from_client: bool | None = None,
+        timestamp: float | None = None,
     ):
-        description, lines, error = contentviews.get_message_content_view(viewname, message, flow)
-        if error:
-            self.master.log.error(error)
+        if view_name and view_name.lower() == "auto":
+            view_name = "auto"
+        pretty = contentviews.prettify_message(message, flow, view_name=view_name)
         if max_lines:
-            lines = islice(lines, max_lines)
+            pretty.text = cut_after_n_lines(pretty.text, max_lines)
 
-        return dict(
-            lines=list(lines),
-            description=description,
+        ret: dict[str, Any] = dict(
+            text=pretty.text,
+            view_name=pretty.view_name,
+            syntax_highlight=pretty.syntax_highlight,
+            description=pretty.description,
         )
+        if from_client is not None:
+            ret["from_client"] = from_client
+        if timestamp is not None:
+            ret["timestamp"] = timestamp
+        return ret
 
-    def get(self, flow_id, message, content_view):
+    def get(self, flow_id, message, content_view) -> None:
         flow = self.flow
-        assert isinstance(flow, (HTTPFlow, TCPFlow))
+        assert isinstance(flow, (HTTPFlow, TCPFlow, UDPFlow))
 
         if self.request.arguments.get("lines"):
             max_lines = int(self.request.arguments["lines"][0])
@@ -487,20 +632,27 @@ class FlowContentView(RequestHandler):
             max_lines = None
 
         if message == "messages":
+            messages: list[TCPMessage] | list[UDPMessage] | list[WebSocketMessage]
             if isinstance(flow, HTTPFlow) and flow.websocket:
                 messages = flow.websocket.messages
-            elif isinstance(flow, TCPFlow):
+            elif isinstance(flow, (TCPFlow, UDPFlow)):
                 messages = flow.messages
             else:
                 raise APIError(400, f"This flow has no messages.")
             msgs = []
             for m in messages:
-                d = self.message_to_json(content_view, m, flow, max_lines)
-                d["from_client"] = m.from_client
-                d["timestamp"] = m.timestamp
+                d = self.message_to_json(
+                    view_name=content_view,
+                    message=m,
+                    flow=flow,
+                    max_lines=max_lines,
+                    from_client=m.from_client,
+                    timestamp=m.timestamp,
+                )
                 msgs.append(d)
                 if max_lines:
-                    max_lines -= len(d["lines"])
+                    max_lines -= d["text"].count("\n") + 1
+                    assert max_lines is not None
                     if max_lines <= 0:
                         break
             self.write(msgs)
@@ -512,7 +664,7 @@ class FlowContentView(RequestHandler):
 class Commands(RequestHandler):
     def get(self) -> None:
         commands = {}
-        for (name, cmd) in self.master.commands.commands.items():
+        for name, cmd in self.master.commands.commands.items():
             commands[name] = {
                 "help": cmd.help,
                 "parameters": [
@@ -523,7 +675,9 @@ class Commands(RequestHandler):
                     }
                     for param in cmd.parameters
                 ],
-                "return_type": command.typename(cmd.return_type) if cmd.return_type else None,
+                "return_type": command.typename(cmd.return_type)
+                if cmd.return_type
+                else None,
                 "signature_help": cmd.signature_help(),
             }
         self.write(commands)
@@ -533,20 +687,20 @@ class ExecuteCommand(RequestHandler):
     def post(self, cmd: str):
         # TODO: We should parse query strings here, this API is painful.
         try:
-            args = self.json['arguments']
+            args = self.json["arguments"]
         except APIError:
             args = []
         try:
             result = self.master.commands.call_strings(cmd, args)
         except Exception as e:
-            self.write({
-                "error": str(e)
-            })
+            self.write({"error": str(e)})
         else:
-            self.write({
-                "value": result,
-                # "type": command.typename(type(result)) if result is not None else "none"
-            })
+            self.write(
+                {
+                    "value": result,
+                    # "type": command.typename(type(result)) if result is not None else "none"
+                }
+            )
 
 
 class Events(RequestHandler):
@@ -575,70 +729,114 @@ class SaveOptions(RequestHandler):
         pass
 
 
-class DnsRebind(RequestHandler):
-    def get(self):
-        raise tornado.web.HTTPError(
-            403,
-            reason="To protect against DNS rebinding, mitmweb can only be accessed by IP at the moment. "
-                   "(https://github.com/mitmproxy/mitmproxy/issues/3234)"
-        )
-
-
-class Conf(RequestHandler):
-    def get(self):
-        conf = {
-            "static": False,
+class State(RequestHandler):
+    # Separate method for testability.
+    @staticmethod
+    def get_json(master: mitmproxy.tools.web.master.WebMaster):
+        return {
             "version": version.VERSION,
-            "contentViews": [v.name for v in contentviews.views if v.name != "Query"]
+            "contentViews": [
+                v for v in contentviews.registry.available_views() if v != "query"
+            ],
+            "servers": {
+                s.mode.full_spec: s.to_json() for s in master.proxyserver.servers
+            },
+            "platform": sys.platform,
+            "localModeUnavailable": mitmproxy_rs.local.LocalRedirector.unavailable_reason(),
         }
-        self.write(f"MITMWEB_CONF = {json.dumps(conf)};")
-        self.set_header("content-type", "application/javascript")
+
+    def get(self):
+        self.write(State.get_json(self.master))
+
+
+class ProcessList(RequestHandler):
+    @staticmethod
+    def get_json():
+        processes = mitmproxy_rs.process_info.active_executables()
+        return [
+            {
+                "is_visible": process.is_visible,
+                "executable": str(process.executable),
+                "is_system": process.is_system,
+                "display_name": process.display_name,
+            }
+            for process in processes
+        ]
+
+    def get(self):
+        self.write(ProcessList.get_json())
+
+
+class ProcessImage(RequestHandler):
+    def get(self):
+        path = self.get_query_argument("path", None)
+
+        if not path:
+            raise APIError(400, "Missing 'path' parameter.")
+
+        try:
+            icon_bytes = mitmproxy_rs.process_info.executable_icon(path)
+        except Exception:
+            icon_bytes = TRANSPARENT_PNG
+
+        self.set_header("Content-Type", "image/png")
+        self.set_header("X-Content-Type-Options", "nosniff")
+        self.set_header("Cache-Control", "max-age=604800")
+        self.write(icon_bytes)
+
+
+class GZipContentAndFlowFiles(tornado.web.GZipContentEncoding):
+    CONTENT_TYPES = {
+        "application/octet-stream",
+        *tornado.web.GZipContentEncoding.CONTENT_TYPES,
+    }
+
+
+handlers = [
+    (r"/", IndexHandler),
+    (r"/filter-help(?:\.json)?", FilterHelp),
+    (r"/updates", ClientConnection),
+    (r"/commands(?:\.json)?", Commands),
+    (r"/commands/(?P<cmd>[a-z.]+)", ExecuteCommand),
+    (r"/events(?:\.json)?", Events),
+    (r"/flows(?:\.json)?", Flows),
+    (r"/flows/dump", DumpFlows),
+    (r"/flows/resume", ResumeFlows),
+    (r"/flows/kill", KillFlows),
+    (r"/flows/(?P<flow_id>[0-9a-f\-]+)", FlowHandler),
+    (r"/flows/(?P<flow_id>[0-9a-f\-]+)/resume", ResumeFlow),
+    (r"/flows/(?P<flow_id>[0-9a-f\-]+)/kill", KillFlow),
+    (r"/flows/(?P<flow_id>[0-9a-f\-]+)/duplicate", DuplicateFlow),
+    (r"/flows/(?P<flow_id>[0-9a-f\-]+)/replay", ReplayFlow),
+    (r"/flows/(?P<flow_id>[0-9a-f\-]+)/revert", RevertFlow),
+    (r"/flows/(?P<flow_id>[0-9a-f\-]+)/(?P<message>request|response|messages)/content.data", FlowContent),
+    (r"/flows/(?P<flow_id>[0-9a-f\-]+)/(?P<message>request|response|messages)/content/(?P<content_view>[0-9a-zA-Z\-\_%]+)(?:\.json)?", FlowContentView),
+    (r"/clear", ClearAll),
+    (r"/options(?:\.json)?", Options),
+    (r"/options/save", SaveOptions),
+    (r"/state(?:\.json)?", State),
+    (r"/processes", ProcessList),
+    (r"/executable-icon", ProcessImage),
+]  # fmt: skip
 
 
 class Application(tornado.web.Application):
-    master: "mitmproxy.tools.web.master.WebMaster"
+    master: mitmproxy.tools.web.master.WebMaster
 
-    def __init__(self, master: "mitmproxy.tools.web.master.WebMaster", debug: bool) -> None:
+    def __init__(
+        self, master: mitmproxy.tools.web.master.WebMaster, debug: bool
+    ) -> None:
         self.master = master
+        auth_addon: WebAuth = master.addons.get("webauth")
         super().__init__(
-            default_host="dns-rebind-protection",
+            handlers=handlers,  # type: ignore  # https://github.com/tornadoweb/tornado/pull/3455
             template_path=os.path.join(os.path.dirname(__file__), "templates"),
             static_path=os.path.join(os.path.dirname(__file__), "static"),
             xsrf_cookies=True,
-            cookie_secret=os.urandom(256),
+            xsrf_cookie_kwargs=dict(httponly=True, samesite="Strict"),
+            cookie_secret=secrets.token_bytes(32),
             debug=debug,
             autoreload=False,
-        )
-
-        self.add_handlers("dns-rebind-protection", [(r"/.*", DnsRebind)])
-        self.add_handlers(
-            # make mitmweb accessible by IP only to prevent DNS rebinding.
-            r'^(localhost|[0-9.]+|\[[0-9a-fA-F:]+\])$',
-            [
-                (r"/", IndexHandler),
-                (r"/filter-help(?:\.json)?", FilterHelp),
-                (r"/updates", ClientConnection),
-                (r"/commands(?:\.json)?", Commands),
-                (r"/commands/(?P<cmd>[a-z.]+)", ExecuteCommand),
-                (r"/events(?:\.json)?", Events),
-                (r"/flows(?:\.json)?", Flows),
-                (r"/flows/dump", DumpFlows),
-                (r"/flows/resume", ResumeFlows),
-                (r"/flows/kill", KillFlows),
-                (r"/flows/(?P<flow_id>[0-9a-f\-]+)", FlowHandler),
-                (r"/flows/(?P<flow_id>[0-9a-f\-]+)/resume", ResumeFlow),
-                (r"/flows/(?P<flow_id>[0-9a-f\-]+)/kill", KillFlow),
-                (r"/flows/(?P<flow_id>[0-9a-f\-]+)/duplicate", DuplicateFlow),
-                (r"/flows/(?P<flow_id>[0-9a-f\-]+)/replay", ReplayFlow),
-                (r"/flows/(?P<flow_id>[0-9a-f\-]+)/revert", RevertFlow),
-                (r"/flows/(?P<flow_id>[0-9a-f\-]+)/(?P<message>request|response|messages)/content.data", FlowContent),
-                (
-                    r"/flows/(?P<flow_id>[0-9a-f\-]+)/(?P<message>request|response|messages)/"
-                    r"content/(?P<content_view>[0-9a-zA-Z\-\_%]+)(?:\.json)?",
-                    FlowContentView),
-                (r"/clear", ClearAll),
-                (r"/options(?:\.json)?", Options),
-                (r"/options/save", SaveOptions),
-                (r"/conf\.js", Conf),
-            ]
+            transforms=[GZipContentAndFlowFiles],
+            is_valid_password=auth_addon.is_valid_password,
         )
